package cz.iivos.pjj.weather;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.server.RemoteServer;
import java.rmi.server.ServerNotActiveException;
import java.rmi.server.UnicastRemoteObject;
import java.util.ArrayList;
import java.util.List;

/**
 * Vzdálený server načítající informace o počasí ze stránky Selected
 * Cities - NOAA's National Weather Service a poskytující tyto
 * informace klientům přes rozhraní RMI pomocí objektů WeatherBean.
 */
public class WeatherServiceImpl extends UnicastRemoteObject
        implements WeatherService {

    /**
     * Načtený seznam objektů s informacemi o počasí v jednotlivých městech.
     */
    private List<WeatherBean> weatherInformation;

    /**
     * Inicializuje server a načte informace o počasí ze stránky Selected Cities -
     * NOAA's National Weather Service
     *
     * @throws RemoteException
     *             pokud při inicializaci serveru dojde k chybě.
     */
    public WeatherServiceImpl() throws RemoteException {
        // exportuje objekt - čeká na spojení na anonymním portu
        super();
        // aktualizuje data ze serveru
        updateWeatherConditions();
    }

    /**
     * Pomocná metoda načítající data o počasí.
     */
    private void updateWeatherConditions() {
        try {
            System.out.println("Aktualizuji informace o počasí...");

            // Selected Cities
            URL url = new URL("http://old.chmi.cz/meteo/om/inform/ppe.html");

            // vytvoření BufferedReaderu pro načtení obsahu webové stránky
            BufferedReader in = new BufferedReader(new InputStreamReader(url.openStream(),"windows-1250"));

            // řetězce reprezentují hlavičky na webové stránce
            // uvozující počasí ve dne a v noci
            String header =
                    "-----------------------------------------------------";
            String inputLine = "";

            // vytvoření seznamu
            weatherInformation = new ArrayList<WeatherBean>();

            for (;;) {
                // nalezení hlavičky, kterou začínají informace o počasí
                do {
                    inputLine = in.readLine();
                } while (inputLine != null && !inputLine.equals(header));

                if (inputLine == null) {
                    // konec vstupu = stránky
                    break;
                }
                // načtení dat
                readBlokOfData(in);
            }
            in.close();

            System.out.println("Informace o počasí aktualizovány.");
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(1);
        }
    }

    /**
     * Pomocná metoda, která načte obsah jednoho bloku dat ze vstupu input do instanční
     * proměnné weatherInformation.
     *
     * @param input
     *            vstup, ze kterého se čtou data.
     * @throws IOException
     *             pokud dojde při čtení dat k chybě.
     */
    private void readBlokOfData(BufferedReader input) throws IOException {
        String inputLine;
        while ((inputLine = input.readLine()).length() > 1) {
            if(inputLine.contains("Hlášení není k dispozici")) {
                continue;
            }
            String city = inputLine.substring(0, 14).trim();
            String cond = inputLine.substring(27, 42).trim();
            if(inputLine.substring(27,28).equals("-")) {
                cond = inputLine.substring(15, 26).trim();
            }
            String temp = inputLine.substring(43, 47).trim();
            WeatherBean weather = new WeatherBean(city, cond, temp);
            weatherInformation.add(weather);
        }
    }

    public List<WeatherBean> getWeatherInformation() throws RemoteException {
        try {
            System.out.println("Zpracovávám dotaz od klienta: "
                    + RemoteServer.getClientHost());
        } catch (ServerNotActiveException e) {
            throw new RuntimeException("Tohle se nestane.", e);
        }
        return weatherInformation;
    }

    public static void main(String[] args) throws Exception {
        System.out.println("Inicializace WeatherService...");

        // vytvoření vzdáleného objektu
        WeatherServiceImpl service = new WeatherServiceImpl();
        // určení adresy vzáleného objektu
        String serverObjectName = "rmi://localhost/WeatherService";
        // svázání objektu s adresou v RMI registry (musí již běžet)
        Naming.rebind(serverObjectName, service);

        System.out.println("WeatherService běží.");
    }
}
